#pragma once
#include <vector>
#include <sstream>
#include <fstream>
#include <J3ML/LinearAlgebra.hpp>
#include <J3ML/LinearAlgebra/Vector3.hpp>
#include<J3ML/Geometry.hpp>
#include <Collage/types/textureInfo.h>

typedef Vector3 Vertex;

enum struct ModelType : uint8_t {
    WAVEFRONT_OBJ = 0,
    ANIMATED_WAVEFRONT_OBJ = 1,
    FBX = 2, //More complicated.
    COLLADA = 3, //*Very* complicated.
};

struct FaceIndices {
    unsigned int vertexIndex[3];
    unsigned int texCoordIndex[3];
};

class SkeletalVertex : public Vertex {
    int8_t joints[3]; //The index of 4 bones that can effect the vertex. 0 represents an empty slot. -1 represents the root bone.
    Vector4 weights; //The 4 weights must total to 1.0
};

class Model {
protected:
    void load(const std::string& file);
    void loadOBJ(const std::string& filename);
    std::vector<Vertex> vertices;
    std::vector<unsigned int> indices;
    std::vector<TextureInformation> textureInfo;
public:
    ModelType type;
    const std::vector<Vertex>& getVertices();
    const std::vector<unsigned int>& getIndices();
    const std::vector<TextureInformation>& getTextureInformation();
    Model() = default;
    explicit Model(const std::string& file);
};

class SkeletalModel : public Model {
protected:
    std::vector<SkeletalVertex> vertices;
public:
    std::vector<SkeletalVertex>& getSkeletalVertices();
};